local entryX, entryY
local exitX, exitY
local solutionPath = {}

joystick = love.joystick.getJoysticks()[1]
require ("gamepadinput")

-- Define player properties
local player = {
    x = 0,
    y = 0,
    speed = 30 -- Increase player speed
}

function love.load()
    love.window.setTitle("Recursive Backtracking Maze")
    math.randomseed(os.time())
    math.random(); math.random(); math.random()
    -- set map width and height, should be odd numbers
    mapWidth = 131
    mapHeight = 81
    tileSize = 8
    -- resize window to maze
    love.window.setMode((mapWidth+2)*tileSize,(mapHeight+2)*tileSize)
    generateMap()

    -- Set player's initial position to entry point
    player.x = entryX * tileSize
    player.y = entryY * tileSize
end

leftxaxis,leftyaxis= 0,0
thumbstickDeadzone=0.2

function love.update(dt)
        
    isjoystickbeingpressed(joystick,button)
    updategetjoystickaxis(dt)
    --print(leftxaxis)
    -- Move player based on thumbstick input
    if leftxaxis > thumbstickDeadzone or leftxaxis < -thumbstickDeadzone then
        movePlayer(leftxaxis * player.speed * dt, 0)
    end
    if leftyaxis > thumbstickDeadzone or leftyaxis < -thumbstickDeadzone then
        movePlayer(0, leftyaxis * player.speed * dt)
    end

end


function love.keypressed(key)
    -- Handle keyboard controls
    if key == "up" then
        movePlayer(0, -player.speed)
    elseif key == "down" then
        movePlayer(0, player.speed)
    elseif key == "left" then
        movePlayer(-player.speed, 0)
    elseif key == "right" then
        movePlayer(player.speed, 0)
    end
end

function love.gamepadpressed(joystick, button)
    -- Handle gamepad controls
    if button == "dpup" then
        movePlayer(0, -player.speed)
    elseif button == "dpdown" then
        movePlayer(0, player.speed)
    elseif button == "dpleft" then
        movePlayer(-player.speed, 0)
    elseif button == "dpright" then
        movePlayer(player.speed, 0)
    end
end

function movePlayer(dx, dy)
    -- Move player if the new position is within the maze boundaries and not a wall
    local newX = player.x + dx
    local newY = player.y + dy

    -- Check if the new position is within the maze boundaries and not a wall
    if newX >= 0 and newX < (mapWidth + 2) * tileSize and
       newY >= 0 and newY < (mapHeight + 2) * tileSize and
       map[math.floor(newX / tileSize) + 1][math.floor(newY / tileSize) + 1] == 1 then
        player.x = newX
        player.y = newY
    end
end

-- Add provided functions for joystick input handling

function updategetjoystickaxis(dt)
    -- Check if a joystick is connected
    if joystick ~= nil then
        -- Get values of left joystick axes
        leftxaxis = joystick:getGamepadAxis("leftx")
        leftyaxis = joystick:getGamepadAxis("lefty")
    end
end
function generateMap()
    -- make a map full of walls
    map = {}
    for x = 1, mapWidth do
        map[x] = {}
        for y = 1, mapHeight do
            map[x][y] = 0
        end
    end
    -- make the maze
    exitX, exitY = ensureExit() -- Capture exit coordinates
    -- randomly select entry point
    entryX, entryY = selectEntryPoint()
    generateMaze()
end



function generateMaze()
    -- make maze using the recursive backtracker algorithm
    -- set and dig the starting space
    local sx, sy = math.random(1,math.floor(mapWidth/2)) * 2, math.random(1, math.floor(mapHeight/2)) * 2
    stack = {{sx, sy}}
    map[sx][sy] = 1
    local neighbours, tx, ty
    while #stack > 0 do
        tx, ty = stack[#stack][1], stack[#stack][2]
        neighbours = getEligibleNeighbours(tx, ty)
        if #neighbours > 0 then
            -- we've got moves, take the first and dig!
            -- first the intermediate space
            tx = tx + neighbours[1][1]
            ty = ty + neighbours[1][2]
            map[tx][ty] = 1
            -- then the destination space
            tx = tx + neighbours[1][1]
            ty = ty + neighbours[1][2]
            map[tx][ty] = 1
            -- add the tile to the end of the stack
            table.insert(stack, {tx, ty})
        else
            -- no moves, remove the last entry on the stack
            table.remove(stack)
        end
    end

    -- Debugging: Print maze structure
    --print("Generated Maze Structure:")
    for y = 1, mapHeight do
        local row = ""
        for x = 1, mapWidth do
            row = row .. map[x][y] .. " "
        end
        --print(row)
    end
end


function ensureExit()
    -- Ensure there is an exit on the edges
    local edge = math.random(4) -- Determine which edge to place the exit on (1: top, 2: right, 3: bottom, 4: left)
    local exitX, exitY
    if edge == 1 then
        -- top edge
        exitX = math.random(1, mapWidth)
        map[exitX][1] = 1
        exitY = 1
    elseif edge == 2 then
        -- right edge
        exitY = math.random(1, mapHeight)
        map[mapWidth][exitY] = 1
        exitX = mapWidth
    elseif edge == 3 then
        -- bottom edge
        exitX = math.random(1, mapWidth)
        map[exitX][mapHeight] = 1
        exitY = mapHeight
    else
        -- left edge
        exitY = math.random(1, mapHeight)
        map[1][exitY] = 1
        exitX = 1
    end
    return exitX, exitY
end


function selectEntryPoint()
    -- Randomly select an entry point
    local edge = math.random(4)
    if edge == 1 then
        -- top edge
        return math.random(1, mapWidth), 1
    elseif edge == 2 then
        -- right edge
        return mapWidth, math.random(1, mapHeight)
    elseif edge == 3 then
        -- bottom edge
        return math.random(1, mapWidth), mapHeight
    else
        -- left edge
        return 1, math.random(1, mapHeight)
    end
end

function solveMaze()
    -- Initialize path variable
    local path = {}
    
    -- Initialize solution path locally
    local solutionPath = {}
    
    -- Debugging: Print entry and exit coordinates
    --print("Entry:", entryX, entryY)
    --print("Exit:", exitX, exitY)
    
    -- Find the solution path
    local visited = {}
    for x = 1, mapWidth do
        visited[x] = {}
        for y = 1, mapHeight do
            visited[x][y] = false
        end
    end
    local queue = {}
    table.insert(queue, {entryX, entryY})
    visited[entryX][entryY] = true
    while #queue > 0 do
        local current = table.remove(queue, 1)
        local x, y = current[1], current[2]
        -- Debugging: Print current position
        --print("Current position:", x, y)
        if x == exitX and y == exitY then
            -- Backtrack to construct the solution path
            -- Ensure entry point is added to the solution path
            --print("Reached exit. Backtracking to construct solution path...")
            table.insert(solutionPath, 1, {entryX, entryY})
            --print("Entry point:", entryX, entryY)
            while x ~= entryX or y ~= entryY do
                -- Debugging: Print the current position during backtracking
                --print("Backtracking - Current position:", x, y)
                -- Insert the current position into the solution path
                table.insert(solutionPath, 1, {x, y}) -- Insert at the beginning of the table
                -- Debugging: Print the position being inserted into the solution path
                --print("Backtracking - Inserted position:", x, y)
                local prevX, prevY = x, y
                local direction = path[prevX][prevY]
                --print("Direction to backtrack:", direction and direction[1] or "nil", direction and direction[2] or "nil")
                x, y = prevX - direction[1], prevY - direction[2]
                -- Debugging: Print the previous position during backtracking
                --print("Backtracking - Previous position:", prevX, prevY)
                -- Debugging: Print the exit condition check
                --print("Backtracking - Exit condition:", x ~= entryX or y ~= entryY) -- Changed to consistent condition
            end
            -- Insert the entry point
            table.insert(solutionPath, 1, {entryX, entryY})
            break
        end
        local neighbours = getEligibleNeighbours(x, y)
        for _, neighbour in ipairs(neighbours) do
            local nx, ny = x + neighbour[1], y + neighbour[2]
            if not visited[nx][ny] then
                -- Debugging: Print the neighbor's position and whether it's visited
                --print("Neighbor position:", nx, ny)
                --print("Visited:", visited[nx][ny])
                visited[nx][ny] = true
                table.insert(queue, {nx, ny})
                -- Store the direction to backtrack
                path[nx] = path[nx] or {}
                path[nx][ny] = {-neighbour[1], -neighbour[2]}
            end
        end
    end
    -- Debugging: Print solution path
    print("Solution Path:", #solutionPath)
end







function getEligibleNeighbours(x, y)
    -- gets if a space two spaces away is legal and if it's a wall
    -- if both adds a dx, dy to neighbours (only +/-1 as we'll use it twice when we dig)
    local neighbours = {}
    -- north
    if y > 2 then
        if map[x][y-2] == 0 then
            table.insert(neighbours, {0, -1})
        end
    end
    -- east
    if x < mapWidth - 1 then
        if map[x+2][y] == 0 then
            table.insert(neighbours, {1, 0})
        end
    end
    -- south
    if y < mapHeight - 1 then
        if map[x][y+2] == 0 then
            table.insert(neighbours, {0, 1})
        end
    end
    -- west
    if x > 2 then
        if map[x-2][y] == 0 then
            table.insert(neighbours, {-1, 0})
        end
    end
    -- shuffle the array if we have more than one elgible
    -- five times should be fine we're only going to have 2 or 3 entries at most
    if #neighbours > 1 then
        shuffleArray(neighbours, 5)
    end
    return neighbours
end

-- quick and dirty array shuffle
function shuffleArray(array, num)
    local s1, s2
    for _ = 1, num do
        s1 = math.random(#array)
        s2 = math.random(#array)
        array[s1], array[s2] = array[s2], array[s1]
    end
end



function love.draw()
    -- Draw maze
    for x = 1, mapWidth do
        for y = 1, mapHeight do
            if map[x][y] == 1 then
                love.graphics.setColor(0.7, 0.7, 0.7)
            else
                love.graphics.setColor(0.2, 0.2, 0.2)
            end
            love.graphics.rectangle("fill", x * tileSize, y * tileSize, tileSize, tileSize)
        end
    end
    
    -- Draw player
    love.graphics.setColor(0, 0, 1) -- Blue color for player
    love.graphics.rectangle("fill", player.x, player.y, tileSize, tileSize)
    
    -- Draw entry and exit points
    love.graphics.setColor(0, 1, 0) -- Green color for entry point
    love.graphics.rectangle("fill", entryX * tileSize, entryY * tileSize, tileSize, tileSize)
    love.graphics.setColor(1, 0, 0) -- Red color for exit point
    love.graphics.rectangle("fill", exitX * tileSize, exitY * tileSize, tileSize, tileSize)
    
    -- Debugging: Print player's position
    --print("Player Position:", player.x, player.y)
    -- Debugging: Print maze boundaries
    --print("Maze Boundaries:", (mapWidth + 2) * tileSize, (mapHeight + 2) * tileSize)
end



